# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.types.admin.organization import (
    UsageCostsResponse,
    UsageImagesResponse,
    UsageEmbeddingsResponse,
    UsageCompletionsResponse,
    UsageModerationsResponse,
    UsageVectorStoresResponse,
    UsageAudioSpeechesResponse,
    UsageWebSearchCallsResponse,
    UsageFileSearchCallsResponse,
    UsageAudioTranscriptionsResponse,
    UsageCodeInterpreterSessionsResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestUsage:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_audio_speeches(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.audio_speeches(
            start_time=0,
        )
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    def test_method_audio_speeches_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.audio_speeches(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_audio_speeches(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.audio_speeches(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_audio_speeches(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.audio_speeches(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_audio_transcriptions(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.audio_transcriptions(
            start_time=0,
        )
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    def test_method_audio_transcriptions_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.audio_transcriptions(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_audio_transcriptions(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.audio_transcriptions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_audio_transcriptions(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.audio_transcriptions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_code_interpreter_sessions(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.code_interpreter_sessions(
            start_time=0,
        )
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    def test_method_code_interpreter_sessions_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.code_interpreter_sessions(
            start_time=0,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_code_interpreter_sessions(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.code_interpreter_sessions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_code_interpreter_sessions(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.code_interpreter_sessions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_completions(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.completions(
            start_time=0,
        )
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    def test_method_completions_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.completions(
            start_time=0,
            api_key_ids=["string"],
            batch=True,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_completions(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.completions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_completions(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.completions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_costs(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.costs(
            start_time=0,
        )
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    def test_method_costs_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.costs(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1d",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_costs(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.costs(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_costs(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.costs(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageCostsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_embeddings(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.embeddings(
            start_time=0,
        )
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    def test_method_embeddings_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.embeddings(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_embeddings(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.embeddings(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_embeddings(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.embeddings(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_file_search_calls(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.file_search_calls(
            start_time=0,
        )
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_method_file_search_calls_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.file_search_calls(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
            user_ids=["string"],
            vector_store_ids=["string"],
        )
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_file_search_calls(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.file_search_calls(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_file_search_calls(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.file_search_calls(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_images(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.images(
            start_time=0,
        )
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    def test_method_images_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.images(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            sizes=["256x256"],
            sources=["image.generation"],
            user_ids=["string"],
        )
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_images(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.images(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_images(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.images(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageImagesResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_moderations(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.moderations(
            start_time=0,
        )
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    def test_method_moderations_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.moderations(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_moderations(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.moderations(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_moderations(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.moderations(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageModerationsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_vector_stores(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.vector_stores(
            start_time=0,
        )
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    def test_method_vector_stores_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.vector_stores(
            start_time=0,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_vector_stores(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.vector_stores(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_vector_stores(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.vector_stores(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_web_search_calls(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.web_search_calls(
            start_time=0,
        )
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_method_web_search_calls_with_all_params(self, client: OpenAI) -> None:
        usage = client.admin.organization.usage.web_search_calls(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            context_levels=["low"],
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_raw_response_web_search_calls(self, client: OpenAI) -> None:
        response = client.admin.organization.usage.with_raw_response.web_search_calls(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    def test_streaming_response_web_search_calls(self, client: OpenAI) -> None:
        with client.admin.organization.usage.with_streaming_response.web_search_calls(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = response.parse()
            assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncUsage:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_audio_speeches(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.audio_speeches(
            start_time=0,
        )
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    async def test_method_audio_speeches_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.audio_speeches(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_audio_speeches(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.audio_speeches(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_audio_speeches(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.audio_speeches(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageAudioSpeechesResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_audio_transcriptions(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.audio_transcriptions(
            start_time=0,
        )
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    async def test_method_audio_transcriptions_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.audio_transcriptions(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_audio_transcriptions(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.audio_transcriptions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_audio_transcriptions(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.audio_transcriptions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageAudioTranscriptionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_code_interpreter_sessions(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.code_interpreter_sessions(
            start_time=0,
        )
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    async def test_method_code_interpreter_sessions_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.code_interpreter_sessions(
            start_time=0,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_code_interpreter_sessions(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.code_interpreter_sessions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_code_interpreter_sessions(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.code_interpreter_sessions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageCodeInterpreterSessionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_completions(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.completions(
            start_time=0,
        )
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    async def test_method_completions_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.completions(
            start_time=0,
            api_key_ids=["string"],
            batch=True,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_completions(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.completions(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_completions(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.completions(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageCompletionsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_costs(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.costs(
            start_time=0,
        )
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    async def test_method_costs_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.costs(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1d",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_costs(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.costs(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageCostsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_costs(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.costs(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageCostsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_embeddings(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.embeddings(
            start_time=0,
        )
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    async def test_method_embeddings_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.embeddings(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_embeddings(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.embeddings(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_embeddings(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.embeddings(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageEmbeddingsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_file_search_calls(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.file_search_calls(
            start_time=0,
        )
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_method_file_search_calls_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.file_search_calls(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
            user_ids=["string"],
            vector_store_ids=["string"],
        )
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_file_search_calls(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.file_search_calls(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_file_search_calls(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.file_search_calls(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageFileSearchCallsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_images(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.images(
            start_time=0,
        )
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    async def test_method_images_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.images(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            sizes=["256x256"],
            sources=["image.generation"],
            user_ids=["string"],
        )
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_images(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.images(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageImagesResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_images(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.images(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageImagesResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_moderations(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.moderations(
            start_time=0,
        )
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    async def test_method_moderations_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.moderations(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_moderations(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.moderations(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageModerationsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_moderations(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.moderations(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageModerationsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_vector_stores(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.vector_stores(
            start_time=0,
        )
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    async def test_method_vector_stores_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.vector_stores(
            start_time=0,
            bucket_width="1m",
            end_time=0,
            group_by=["project_id"],
            limit=0,
            page="page",
            project_ids=["string"],
        )
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_vector_stores(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.vector_stores(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_vector_stores(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.vector_stores(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageVectorStoresResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_web_search_calls(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.web_search_calls(
            start_time=0,
        )
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_method_web_search_calls_with_all_params(self, async_client: AsyncOpenAI) -> None:
        usage = await async_client.admin.organization.usage.web_search_calls(
            start_time=0,
            api_key_ids=["string"],
            bucket_width="1m",
            context_levels=["low"],
            end_time=0,
            group_by=["project_id"],
            limit=0,
            models=["string"],
            page="page",
            project_ids=["string"],
            user_ids=["string"],
        )
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_raw_response_web_search_calls(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.admin.organization.usage.with_raw_response.web_search_calls(
            start_time=0,
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        usage = response.parse()
        assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

    @parametrize
    async def test_streaming_response_web_search_calls(self, async_client: AsyncOpenAI) -> None:
        async with async_client.admin.organization.usage.with_streaming_response.web_search_calls(
            start_time=0,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            usage = await response.parse()
            assert_matches_type(UsageWebSearchCallsResponse, usage, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["RealtimeConversationItemFunctionCallParam"]


class RealtimeConversationItemFunctionCallParam(TypedDict, total=False):
    """A function call item in a Realtime conversation."""

    arguments: Required[str]
    """The arguments of the function call.

    This is a JSON-encoded string representing the arguments passed to the function,
    for example `{"arg1": "value1", "arg2": 42}`.
    """

    name: Required[str]
    """The name of the function being called."""

    type: Required[Literal["function_call"]]
    """The type of the item. Always `function_call`."""

    id: str
    """The unique ID of the item.

    This may be provided by the client or generated by the server.
    """

    call_id: str
    """The ID of the function call."""

    object: Literal["realtime.item"]
    """Identifier for the API object being returned - always `realtime.item`.

    Optional when creating a new item.
    """

    status: Literal["completed", "incomplete", "in_progress"]
    """The status of the item. Has no effect on the conversation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ....._models import BaseModel

__all__ = ["OrganizationGroupUser"]


class OrganizationGroupUser(BaseModel):
    """Represents an individual user returned when inspecting group membership."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    email: Optional[str] = None
    """The email address of the user."""

    name: str
    """The name of the user."""



from __future__ import annotations

from typing import Callable, Protocol, Awaitable
from weakref import WeakKeyDictionary
from dataclasses import dataclass

import httpx

from ._models import FinalRequestOptions
from ._exceptions import OpenAIError


class _Provider:
    """Opaque configuration returned by an OpenAI-owned provider factory."""

    __slots__ = ("__weakref__",)


@dataclass
class _ProviderRuntime:
    name: str
    base_url: str | httpx.URL
    transform_request: Callable[[FinalRequestOptions], FinalRequestOptions] | None = None
    transform_async_request: Callable[[FinalRequestOptions], Awaitable[FinalRequestOptions]] | None = None
    prepare_request: Callable[[httpx.Request], None] | None = None
    prepare_async_request: Callable[[httpx.Request], Awaitable[None]] | None = None
    normalize_response: Callable[[httpx.Response], httpx.Response] | None = None
    normalize_async_response: Callable[[httpx.Response], Awaitable[httpx.Response]] | None = None


class _ProviderDefinition(Protocol):
    @property
    def name(self) -> str: ...

    def configure(self) -> _ProviderRuntime: ...


# Provider factories capture configuration in definitions, while every client
# gets fresh runtime state from ``definition.configure()``. Keeping definitions
# outside the opaque provider object prevents arbitrary objects (including a
# directly constructed ``_Provider``) from imitating an OpenAI-owned provider
# and keeps credentials out of the object's representation. The weak mapping
# also avoids retaining provider configuration after the public handle is gone.
_provider_definitions: WeakKeyDictionary[_Provider, _ProviderDefinition] = WeakKeyDictionary()


def _create_provider(definition: _ProviderDefinition) -> _Provider:  # pyright: ignore[reportUnusedFunction]
    provider = _Provider()
    _provider_definitions[provider] = definition
    return provider


def _provider_name(provider: _Provider) -> str:  # pyright: ignore[reportUnusedFunction]
    return _get_provider_definition(provider).name


def _configure_provider(provider: _Provider) -> _ProviderRuntime:  # pyright: ignore[reportUnusedFunction]
    return _get_provider_definition(provider).configure()


def _get_provider_definition(provider: _Provider) -> _ProviderDefinition:
    try:
        return _provider_definitions[provider]
    except (KeyError, TypeError) as exc:
        raise OpenAIError("Invalid provider. Providers must be created by an OpenAI provider factory.") from exc



import json
from typing import cast
from pathlib import Path

import httpx
import respx
import pytest
from respx.models import Call
from inline_snapshot import snapshot

from openai import OpenAI, OAuthError
from openai.auth._workload import (
    gcp_id_token_provider,
    k8s_service_account_token_provider,
    azure_managed_identity_token_provider,
)


@respx.mock
def test_basic_auth():
    respx.post("https://auth.openai.com/oauth/token").mock(
        return_value=httpx.Response(
            200,
            json={
                "access_token": "fake_access_token",
                "issued_token_type": "urn:ietf:params:oauth:token-type:access_token",
                "token_type": "Bearer",
                "expires_in": 3600,
            },
        )
    )

    respx.get("https://api.openai.com/v1/models").mock(
        return_value=httpx.Response(200, json={"data": [], "object": "list"})
    )

    client = OpenAI(
        workload_identity={
            "identity_provider_id": "idp_123",
            "service_account_id": "sa_123",
            "provider": {
                "get_token": lambda: "fake_subject_token",
                "token_type": "jwt",
            },
        },
    )

    client.models.list()

    assert len(respx.calls) == 2
    token_call = cast(Call, respx.calls[0])
    api_call = cast(Call, respx.calls[1])

    assert token_call.request.url == "https://auth.openai.com/oauth/token"
    assert api_call.request.headers.get("Authorization") == "Bearer fake_access_token"


@respx.mock
def test_workload_identity_exchange_payload_and_cache() -> None:
    provider_call_count = 0

    def provider() -> str:
        nonlocal provider_call_count
        provider_call_count += 1
        return "fake_subject_token"

    exchange_route = respx.post("https://auth.openai.com/oauth/token").mock(
        return_value=httpx.Response(
            200,
            json={
                "access_token": "fake_access_token",
                "issued_token_type": "urn:ietf:params:oauth:token-type:access_token",
                "token_type": "Bearer",
                "expires_in": 3600,
            },
        )
    )
    api_route = respx.get("https://api.openai.com/v1/models").mock(
        return_value=httpx.Response(200, json={"data": [], "object": "list"})
    )

    client = OpenAI(
        workload_identity={
            "identity_provider_id": "idp_123",
            "service_account_id": "sa_123",
            "provider": {
                "get_token": provider,
                "token_type": "jwt",
            },
        },
    )

    client.models.list()
    client.models.list()

    assert provider_call_count == 1
    assert exchange_route.call_count == 1
    assert api_route.call_count == 2

    exchange_request = cast(respx.models.Call, exchange_route.calls[0]).request
    assert json.loads(exchange_request.content) == snapshot(
        {
            "grant_type": "urn:ietf:params:oauth:grant-type:token-exchange",
            "subject_token": "fake_subject_token",
            "subject_token_type": "urn:ietf:params:oauth:token-type:jwt",
            "identity_provider_id": "idp_123",
            "service_account_id": "sa_123",
        }
    )

    assert (
        cast(respx.models.Call, api_route.calls[0]).request.headers.get("Authorization") == "Bearer fake_access_token"
    )
    assert (
        cast(respx.models.Call, api_route.calls[1]).request.headers.get("Authorization") == "Bearer fake_access_token"
    )


@respx.mock
def test_workload_identity_exchange_error() -> None:
    exchange_route = respx.post("https://auth.openai.com/oauth/token").mock(
        return_value=httpx.Response(
            401,
            json={
                "error": "invalid_grant",
                "error_description": "No service account mapping found for the provided service_account_id.",
            },
        )
    )
    api_route = respx.get("https://api.openai.com/v1/models").mock(
        return_value=httpx.Response(200, json={"data": [], "object": "list"})
    )

    client = OpenAI(
        workload_identity={
            "identity_provider_id": "idp_123",
            "service_account_id": "sa_123",
            "provider": {
                "get_token": lambda: "fake_subject_token",
                "token_type": "jwt",
            },
        },
    )

    with pytest.raises(OAuthError) as exc:
        client.models.list()

    assert exc.value.message == "No service account mapping found for the provided service_account_id."
    assert exc.value.error == "invalid_grant"
    assert exc.value.status_code == 401
    assert exchange_route.call_count == 1
    assert api_route.call_count == 0


def test_k8s_service_account_token_provider(tmp_path: Path) -> None:
    token_file = tmp_path / "token"
    token_file.write_text("my-k8s-token")

    provider = k8s_service_account_token_provider(token_file)

    assert provider["token_type"] == "jwt"
    assert provider["get_token"]() == "my-k8s-token"


@respx.mock
def test_azure_managed_identity_token_provider() -> None:
    respx.get("http://169.254.169.254/metadata/identity/oauth2/token").mock(
        return_value=httpx.Response(200, json={"access_token": "azure-token"})
    )

    provider = azure_managed_identity_token_provider()

    assert provider["token_type"] == "jwt"
    assert provider["get_token"]() == "azure-token"


@respx.mock
def test_gcp_id_token_provider() -> None:
    respx.get("http://metadata.google.internal/computeMetadata/v1/instance/service-accounts/default/identity").mock(
        return_value=httpx.Response(200, text="gcp-token")
    )

    provider = gcp_id_token_provider()

    assert provider["token_type"] == "id"
    assert provider["get_token"]() == "gcp-token"



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable, Optional
from typing_extensions import Literal, Required, Annotated, TypeAlias, TypedDict

from ...._utils import PropertyInfo
from ..beta_tool_param import BetaToolParam
from ..beta_tool_choice_options import BetaToolChoiceOptions
from ..beta_tool_choice_mcp_param import BetaToolChoiceMcpParam
from ..beta_tool_choice_shell_param import BetaToolChoiceShellParam
from ..beta_tool_choice_types_param import BetaToolChoiceTypesParam
from ..beta_tool_choice_custom_param import BetaToolChoiceCustomParam
from ..beta_response_input_item_param import BetaResponseInputItemParam
from ..beta_tool_choice_allowed_param import BetaToolChoiceAllowedParam
from ..beta_tool_choice_function_param import BetaToolChoiceFunctionParam
from ..beta_tool_choice_apply_patch_param import BetaToolChoiceApplyPatchParam
from ..beta_response_conversation_param_param import BetaResponseConversationParamParam
from ..beta_response_format_text_config_param import BetaResponseFormatTextConfigParam

__all__ = [
    "InputTokenCountParams",
    "Conversation",
    "Reasoning",
    "Text",
    "ToolChoice",
    "ToolChoiceBetaSpecificProgrammaticToolCallingParam",
]


class InputTokenCountParams(TypedDict, total=False):
    conversation: Optional[Conversation]
    """The conversation that this response belongs to.

    Items from this conversation are prepended to `input_items` for this response
    request. Input items and output items from this response are automatically added
    to this conversation after this response completes.
    """

    input: Union[str, Iterable[BetaResponseInputItemParam], None]
    """Text, image, or file inputs to the model, used to generate a response"""

    instructions: Optional[str]
    """
    A system (or developer) message inserted into the model's context. When used
    along with `previous_response_id`, the instructions from a previous response
    will not be carried over to the next response. This makes it simple to swap out
    system (or developer) messages in new responses.
    """

    model: Optional[str]
    """Model ID used to generate the response, like `gpt-4o` or `o3`.

    OpenAI offers a wide range of models with different capabilities, performance
    characteristics, and price points. Refer to the
    [model guide](https://platform.openai.com/docs/models) to browse and compare
    available models.
    """

    parallel_tool_calls: Optional[bool]
    """Whether to allow the model to run tool calls in parallel."""

    personality: Union[str, Literal["friendly", "pragmatic"]]
    """A model-owned style preset to apply to this request.

    Omit this parameter to use the model's default style. Supported values may
    expand over time. Values must be at most 64 characters.
    """

    previous_response_id: Optional[str]
    """The unique ID of the previous response to the model.

    Use this to create multi-turn conversations. Learn more about
    [conversation state](https://platform.openai.com/docs/guides/conversation-state).
    Cannot be used in conjunction with `conversation`.
    """

    reasoning: Optional[Reasoning]
    """
    **gpt-5 and o-series models only** Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    text: Optional[Text]
    """Configuration options for a text response from the model.

    Can be plain text or structured JSON data. Learn more:

    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    tool_choice: Optional[ToolChoice]
    """Controls which tool the model should use, if any."""

    tools: Optional[Iterable[BetaToolParam]]
    """An array of tools the model may call while generating a response.

    You can specify which tool to use by setting the `tool_choice` parameter.
    """

    truncation: Literal["auto", "disabled"]
    """The truncation strategy to use for the model response.

    - `auto`: If the input to this Response exceeds the model's context window size,
      the model will truncate the response to fit the context window by dropping
      items from the beginning of the conversation. - `disabled` (default): If the
      input size will exceed the context window size for a model, the request will
      fail with a 400 error.
    """

    betas: Annotated[List[Literal["responses_multi_agent=v1"]], PropertyInfo(alias="openai-beta")]


Conversation: TypeAlias = Union[str, BetaResponseConversationParamParam]


class Reasoning(TypedDict, total=False):
    """
    **gpt-5 and o-series models only** Configuration options for [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    context: Optional[Literal["auto", "current_turn", "all_turns"]]
    """
    Controls which reasoning items are rendered back to the model on later turns. If
    omitted or set to `auto`, the model determines the context mode. The `gpt-5.6`
    model family defaults to `all_turns`; earlier models default to `current_turn`.

    When returned on a response, this is the effective reasoning context mode used
    for the response.
    """

    effort: Optional[Literal["none", "minimal", "low", "medium", "high", "xhigh", "max"]]
    """Constrains effort on reasoning for reasoning models.

    Currently supported values are `none`, `minimal`, `low`, `medium`, `high`,
    `xhigh`, and `max`. Reducing reasoning effort can result in faster responses and
    fewer tokens used on reasoning in a response. Not all reasoning models support
    every value. See the
    [reasoning guide](https://platform.openai.com/docs/guides/reasoning) for
    model-specific support.
    """

    generate_summary: Optional[Literal["auto", "concise", "detailed"]]
    """**Deprecated:** use `summary` instead.

    A summary of the reasoning performed by the model. This can be useful for
    debugging and understanding the model's reasoning process. One of `auto`,
    `concise`, or `detailed`.
    """

    mode: Union[str, Literal["standard", "pro"]]
    """Controls the reasoning execution mode for the request.

    When returned on a response, this is the effective execution mode.
    """

    summary: Optional[Literal["auto", "concise", "detailed"]]
    """A summary of the reasoning performed by the model.

    This can be useful for debugging and understanding the model's reasoning
    process. One of `auto`, `concise`, or `detailed`.

    `concise` is supported for `computer-use-preview` models and all reasoning
    models after `gpt-5`.
    """


class Text(TypedDict, total=False):
    """Configuration options for a text response from the model.

    Can be plain
    text or structured JSON data. Learn more:
    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    format: BetaResponseFormatTextConfigParam
    """An object specifying the format that the model must output.

    Configuring `{ "type": "json_schema" }` enables Structured Outputs, which
    ensures the model will match your supplied JSON schema. Learn more in the
    [Structured Outputs guide](https://platform.openai.com/docs/guides/structured-outputs).

    The default format is `{ "type": "text" }` with no additional options.

    **Not recommended for gpt-4o and newer models:**

    Setting to `{ "type": "json_object" }` enables the older JSON mode, which
    ensures the message the model generates is valid JSON. Using `json_schema` is
    preferred for models that support it.
    """

    verbosity: Optional[Literal["low", "medium", "high"]]
    """Constrains the verbosity of the model's response.

    Lower values will result in more concise responses, while higher values will
    result in more verbose responses. Currently supported values are `low`,
    `medium`, and `high`. The default is `medium`.
    """


class ToolChoiceBetaSpecificProgrammaticToolCallingParam(TypedDict, total=False):
    type: Required[Literal["programmatic_tool_calling"]]
    """The tool to call. Always `programmatic_tool_calling`."""


ToolChoice: TypeAlias = Union[
    BetaToolChoiceOptions,
    BetaToolChoiceAllowedParam,
    BetaToolChoiceTypesParam,
    BetaToolChoiceFunctionParam,
    BetaToolChoiceMcpParam,
    BetaToolChoiceCustomParam,
    ToolChoiceBetaSpecificProgrammaticToolCallingParam,
    BetaToolChoiceApplyPatchParam,
    BetaToolChoiceShellParam,
]



from __future__ import annotations

import sys
import importlib
from typing import Any, Protocol, cast

import httpx

from ._constants import DEFAULT_TIMEOUT, DEFAULT_CONNECTION_LIMITS


class _Httpx2Module(Protocol):
    Auth: type[httpx.Auth]
    Client: type[httpx.Client]
    AsyncClient: type[httpx.AsyncClient]
    URL: type[httpx.URL]
    Response: type[httpx.Response]
    Timeout: type[httpx.Timeout]
    Limits: type[httpx.Limits]
    TimeoutException: type[httpx.TimeoutException]
    HTTPStatusError: type[httpx.HTTPStatusError]
    StreamConsumed: type[httpx.StreamConsumed]
    RequestNotRead: type[httpx.RequestNotRead]


def _loaded_httpx2() -> _Httpx2Module | None:
    module = sys.modules.get("httpx2")
    if module is None:
        return None
    return cast(_Httpx2Module, module)


def _supports_httpx2() -> bool:
    return sys.version_info >= (3, 10)


def _require_httpx2() -> _Httpx2Module:
    if not _supports_httpx2():
        raise RuntimeError(
            "HTTPX2 requires Python 3.10 or later; install the httpx2 extra on a supported interpreter: "
            "pip install 'openai[httpx2]'"
        )

    try:
        module = importlib.import_module("httpx2")
    except ImportError:
        raise RuntimeError("To use HTTPX2, install the httpx2 extra: pip install 'openai[httpx2]'") from None

    return cast(_Httpx2Module, module)


def is_httpx2_sync_client(value: object) -> bool:
    module = _loaded_httpx2()
    return module is not None and isinstance(value, module.Client)


def is_httpx2_async_client(value: object) -> bool:
    module = _loaded_httpx2()
    return module is not None and isinstance(value, module.AsyncClient)


def normalize_httpx_url(value: str | httpx.URL) -> httpx.URL:
    module = _loaded_httpx2()
    if module is not None and isinstance(value, module.URL):
        return httpx.URL(str(value))
    if isinstance(value, httpx.URL):
        return value
    return httpx.URL(value)


def http_response_types() -> tuple[type[httpx.Response], ...]:
    module = _loaded_httpx2()
    if module is None:
        return (httpx.Response,)
    return (httpx.Response, module.Response)


def normalize_httpx_timeout(value: float | httpx.Timeout | None) -> float | httpx.Timeout | None:
    module = _loaded_httpx2()
    if module is not None and isinstance(value, module.Timeout):
        return httpx.Timeout(**value.as_dict())
    return value


def normalize_httpx2_timeout(value: float | httpx.Timeout | None) -> float | httpx.Timeout | None:
    if isinstance(value, httpx.Timeout):
        return _require_httpx2().Timeout(**value.as_dict())
    return value


def normalize_httpx2_auth(value: httpx.Auth) -> httpx.Auth:
    if type(value) is httpx.Auth:
        return _require_httpx2().Auth()
    return value


def timeout_exceptions() -> tuple[type[httpx.TimeoutException], ...]:
    module = _loaded_httpx2()
    if module is None:
        return (httpx.TimeoutException,)
    return (httpx.TimeoutException, module.TimeoutException)


def status_exceptions() -> tuple[type[httpx.HTTPStatusError], ...]:
    module = _loaded_httpx2()
    if module is None:
        return (httpx.HTTPStatusError,)
    return (httpx.HTTPStatusError, module.HTTPStatusError)


def stream_consumed_exceptions() -> tuple[type[httpx.StreamConsumed], ...]:
    module = _loaded_httpx2()
    if module is None:
        return (httpx.StreamConsumed,)
    return (httpx.StreamConsumed, module.StreamConsumed)


def request_not_read_exceptions() -> tuple[type[httpx.RequestNotRead], ...]:
    module = _loaded_httpx2()
    if module is None:
        return (httpx.RequestNotRead,)
    return (httpx.RequestNotRead, module.RequestNotRead)


def _set_httpx2_defaults(kwargs: dict[str, Any]) -> _Httpx2Module:
    module = _require_httpx2()
    timeout = kwargs.get("timeout", DEFAULT_TIMEOUT)
    kwargs["timeout"] = normalize_httpx2_timeout(timeout)

    limits = kwargs.get("limits", DEFAULT_CONNECTION_LIMITS)
    if isinstance(limits, httpx.Limits):
        kwargs["limits"] = module.Limits(
            max_connections=limits.max_connections,
            max_keepalive_connections=limits.max_keepalive_connections,
            keepalive_expiry=limits.keepalive_expiry,
        )

    kwargs.setdefault("follow_redirects", True)
    return module


def DefaultHttpx2Client(**kwargs: Any) -> httpx.Client:
    """Create an experimental HTTPX2 client with the SDK's recommended defaults."""
    module = _set_httpx2_defaults(kwargs)
    return module.Client(**kwargs)


def DefaultAsyncHttpx2Client(**kwargs: Any) -> httpx.AsyncClient:
    """Create an experimental async HTTPX2 client with the SDK's recommended defaults."""
    module = _set_httpx2_defaults(kwargs)
    return module.AsyncClient(**kwargs)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .beta_tool_param import BetaToolParam
from .beta_response_includable import BetaResponseIncludable
from .beta_tool_choice_options import BetaToolChoiceOptions
from .beta_response_input_param import BetaResponseInputParam
from .beta_response_prompt_param import BetaResponsePromptParam
from .beta_tool_choice_mcp_param import BetaToolChoiceMcpParam
from .beta_tool_choice_shell_param import BetaToolChoiceShellParam
from .beta_tool_choice_types_param import BetaToolChoiceTypesParam
from .beta_tool_choice_custom_param import BetaToolChoiceCustomParam
from .beta_tool_choice_allowed_param import BetaToolChoiceAllowedParam
from .beta_response_text_config_param import BetaResponseTextConfigParam
from .beta_tool_choice_function_param import BetaToolChoiceFunctionParam
from .beta_response_inject_event_param import BetaResponseInjectEventParam
from .beta_tool_choice_apply_patch_param import BetaToolChoiceApplyPatchParam
from .beta_response_conversation_param_param import BetaResponseConversationParamParam

__all__ = [
    "BetaResponsesClientEventParam",
    "ResponseCreate",
    "ResponseCreateContextManagement",
    "ResponseCreateConversation",
    "ResponseCreateModeration",
    "ResponseCreateModerationPolicy",
    "ResponseCreateModerationPolicyInput",
    "ResponseCreateModerationPolicyOutput",
    "ResponseCreateMultiAgent",
    "ResponseCreatePromptCacheOptions",
    "ResponseCreateReasoning",
    "ResponseCreateStreamOptions",
    "ResponseCreateToolChoice",
    "ResponseCreateToolChoiceBetaSpecificProgrammaticToolCallingParam",
]


class ResponseCreateContextManagement(TypedDict, total=False):
    type: Required[str]
    """The context management entry type. Currently only 'compaction' is supported."""

    compact_threshold: Optional[int]
    """Token threshold at which compaction should be triggered for this entry."""


ResponseCreateConversation: TypeAlias = Union[str, BetaResponseConversationParamParam]


class ResponseCreateModerationPolicyInput(TypedDict, total=False):
    """The moderation policy for the response input."""

    mode: Required[Literal["score", "block"]]


class ResponseCreateModerationPolicyOutput(TypedDict, total=False):
    """The moderation policy for the response output."""

    mode: Required[Literal["score", "block"]]


class ResponseCreateModerationPolicy(TypedDict, total=False):
    """The policy to apply to moderated response input and output."""

    input: Optional[ResponseCreateModerationPolicyInput]
    """The moderation policy for the response input."""

    output: Optional[ResponseCreateModerationPolicyOutput]
    """The moderation policy for the response output."""


class ResponseCreateModeration(TypedDict, total=False):
    """Configuration for running moderation on the input and output of this response."""

    model: Required[str]
    """The moderation model to use for moderated completions, e.g.

    'omni-moderation-latest'.
    """

    policy: Optional[ResponseCreateModerationPolicy]
    """The policy to apply to moderated response input and output."""


class ResponseCreateMultiAgent(TypedDict, total=False):
    """Configuration for server-hosted multi-agent execution."""

    enabled: Required[bool]
    """Whether to enable server-hosted multi-agent execution for this response."""

    max_concurrent_subagents: int
    """
    `max_concurrent_subagents` sets the maximum number of subagents that can be
    active simultaneously across the entire agent tree. It includes all
    descendants—children, grandchildren, and deeper subagents—but excludes the root
    agent. The API does not impose a fixed upper bound on this setting. The default
    is `3`, which is recommended for most workloads. Multi-agent runs also have no
    fixed limit on tree depth or the total number of subagents created during a run.
    """


class ResponseCreatePromptCacheOptions(TypedDict, total=False):
    """Options for prompt caching.

    Supported for `gpt-5.6` and later models. By default, OpenAI automatically chooses one implicit cache breakpoint. You can add explicit breakpoints to content blocks with `prompt_cache_breakpoint`. Each request can write up to four breakpoints. For cache matching, OpenAI considers up to the latest 80 breakpoints in the conversation, without a content-block lookback limit. Set `mode` to `explicit` to disable the implicit breakpoint. The `ttl` defaults to `30m`, which is currently the only supported value. See the [prompt caching guide](https://platform.openai.com/docs/guides/prompt-caching) for current details.
    """

    mode: Literal["implicit", "explicit"]
    """Controls whether OpenAI automatically creates an implicit cache breakpoint.

    Defaults to `implicit`. With `implicit`, OpenAI creates one implicit breakpoint
    and writes up to the latest three explicit breakpoints in the request. With
    `explicit`, OpenAI does not create an implicit breakpoint and writes up to the
    latest four explicit breakpoints. If there are no explicit breakpoints, the
    request does not use prompt caching.
    """

    ttl: Literal["30m"]
    """
    The minimum lifetime applied to every implicit and explicit cache breakpoint
    written by the request. Defaults to `30m`, which is currently the only supported
    value. The backend may retain cache entries for longer.
    """


class ResponseCreateReasoning(TypedDict, total=False):
    """**gpt-5 and o-series models only**

    Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    context: Optional[Literal["auto", "current_turn", "all_turns"]]
    """
    Controls which reasoning items are rendered back to the model on later turns. If
    omitted or set to `auto`, the model determines the context mode. The `gpt-5.6`
    model family defaults to `all_turns`; earlier models default to `current_turn`.

    When returned on a response, this is the effective reasoning context mode used
    for the response.
    """

    effort: Optional[Literal["none", "minimal", "low", "medium", "high", "xhigh", "max"]]
    """Constrains effort on reasoning for reasoning models.

    Currently supported values are `none`, `minimal`, `low`, `medium`, `high`,
    `xhigh`, and `max`. Reducing reasoning effort can result in faster responses and
    fewer tokens used on reasoning in a response. Not all reasoning models support
    every value. See the
    [reasoning guide](https://platform.openai.com/docs/guides/reasoning) for
    model-specific support.
    """

    generate_summary: Optional[Literal["auto", "concise", "detailed"]]
    """**Deprecated:** use `summary` instead.

    A summary of the reasoning performed by the model. This can be useful for
    debugging and understanding the model's reasoning process. One of `auto`,
    `concise`, or `detailed`.
    """

    mode: Union[str, Literal["standard", "pro"]]
    """Controls the reasoning execution mode for the request.

    When returned on a response, this is the effective execution mode.
    """

    summary: Optional[Literal["auto", "concise", "detailed"]]
    """A summary of the reasoning performed by the model.

    This can be useful for debugging and understanding the model's reasoning
    process. One of `auto`, `concise`, or `detailed`.

    `concise` is supported for `computer-use-preview` models and all reasoning
    models after `gpt-5`.
    """


class ResponseCreateStreamOptions(TypedDict, total=False):
    """Options for streaming responses. Only set this when you set `stream: true`."""

    include_obfuscation: bool
    """When true, stream obfuscation will be enabled.

    Stream obfuscation adds random characters to an `obfuscation` field on streaming
    delta events to normalize payload sizes as a mitigation to certain side-channel
    attacks. These obfuscation fields are included by default, but add a small
    amount of overhead to the data stream. You can set `include_obfuscation` to
    false to optimize for bandwidth if you trust the network links between your
    application and the OpenAI API.
    """


class ResponseCreateToolChoiceBetaSpecificProgrammaticToolCallingParam(TypedDict, total=False):
    type: Required[Literal["programmatic_tool_calling"]]
    """The tool to call. Always `programmatic_tool_calling`."""


ResponseCreateToolChoice: TypeAlias = Union[
    BetaToolChoiceOptions,
    BetaToolChoiceAllowedParam,
    BetaToolChoiceTypesParam,
    BetaToolChoiceFunctionParam,
    BetaToolChoiceMcpParam,
    BetaToolChoiceCustomParam,
    ResponseCreateToolChoiceBetaSpecificProgrammaticToolCallingParam,
    BetaToolChoiceApplyPatchParam,
    BetaToolChoiceShellParam,
]


class ResponseCreate(TypedDict, total=False):
    """
    Client event for creating a response over a persistent WebSocket connection.
    This payload uses the same top-level fields as `POST /v1/responses`.

    Notes:
    - `stream` is implicit over WebSocket and should not be sent.
    - `background` is not supported over WebSocket.
    """

    type: Required[Literal["response.create"]]
    """The type of the client event. Always `response.create`."""

    background: Optional[bool]
    """
    Whether to run the model response in the background.
    [Learn more](https://platform.openai.com/docs/guides/background).
    """

    context_management: Optional[Iterable[ResponseCreateContextManagement]]
    """Context management configuration for this request."""

    conversation: Optional[ResponseCreateConversation]
    """The conversation that this response belongs to.

    Items from this conversation are prepended to `input_items` for this response
    request. Input items and output items from this response are automatically added
    to this conversation after this response completes.
    """

    include: Optional[List[BetaResponseIncludable]]
    """Specify additional output data to include in the model response.

    Currently supported values are:

    - `web_search_call.action.sources`: Include the sources of the web search tool
      call.
    - `code_interpreter_call.outputs`: Includes the outputs of python code execution
      in code interpreter tool call items.
    - `computer_call_output.output.image_url`: Include image urls from the computer
      call output.
    - `file_search_call.results`: Include the search results of the file search tool
      call.
    - `message.input_image.image_url`: Include image urls from the input message.
    - `message.output_text.logprobs`: Include logprobs with assistant messages.
    - `reasoning.encrypted_content`: Includes an encrypted version of reasoning
      tokens in reasoning item outputs. This enables reasoning items to be used in
      multi-turn conversations when using the Responses API statelessly (like when
      the `store` parameter is set to `false`, or when an organization is enrolled
      in the zero data retention program).
    """

    input: Union[str, BetaResponseInputParam]
    """Text, image, or file inputs to the model, used to generate a response.

    Learn more:

    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Image inputs](https://platform.openai.com/docs/guides/images)
    - [File inputs](https://platform.openai.com/docs/guides/pdf-files)
    - [Conversation state](https://platform.openai.com/docs/guides/conversation-state)
    - [Function calling](https://platform.openai.com/docs/guides/function-calling)
    """

    instructions: Optional[str]
    """A system (or developer) message inserted into the model's context.

    When using along with `previous_response_id`, the instructions from a previous
    response will not be carried over to the next response. This makes it simple to
    swap out system (or developer) messages in new responses.
    """

    max_output_tokens: Optional[int]
    """
    An upper bound for the number of tokens that can be generated for a response,
    including visible output tokens and
    [reasoning tokens](https://platform.openai.com/docs/guides/reasoning).
    """

    max_tool_calls: Optional[int]
    """
    The maximum number of total calls to built-in tools that can be processed in a
    response. This maximum number applies across all built-in tool calls, not per
    individual tool. Any further attempts to call a tool by the model will be
    ignored.
    """

    metadata: Optional[Dict[str, str]]
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format, and querying for objects via API or the dashboard.

    Keys are strings with a maximum length of 64 characters. Values are strings with
    a maximum length of 512 characters.
    """

    model: Union[
        Literal[
            "gpt-5.6-sol",
            "gpt-5.6-terra",
            "gpt-5.6-luna",
            "gpt-5.4",
            "gpt-5.4-mini",
            "gpt-5.4-nano",
            "gpt-5.4-mini-2026-03-17",
            "gpt-5.4-nano-2026-03-17",
            "gpt-5.3-chat-latest",
            "gpt-5.2",
            "gpt-5.2-2025-12-11",
            "gpt-5.2-chat-latest",
            "gpt-5.2-pro",
            "gpt-5.2-pro-2025-12-11",
            "gpt-5.1",
            "gpt-5.1-2025-11-13",
            "gpt-5.1-codex",
            "gpt-5.1-mini",
            "gpt-5.1-chat-latest",
            "gpt-5",
            "gpt-5-mini",
            "gpt-5-nano",
            "gpt-5-2025-08-07",
            "gpt-5-mini-2025-08-07",
            "gpt-5-nano-2025-08-07",
            "gpt-5-chat-latest",
            "gpt-4.1",
            "gpt-4.1-mini",
            "gpt-4.1-nano",
            "gpt-4.1-2025-04-14",
            "gpt-4.1-mini-2025-04-14",
            "gpt-4.1-nano-2025-04-14",
            "o4-mini",
            "o4-mini-2025-04-16",
            "o3",
            "o3-2025-04-16",
            "o3-mini",
            "o3-mini-2025-01-31",
            "o1",
            "o1-2024-12-17",
            "o1-preview",
            "o1-preview-2024-09-12",
            "o1-mini",
            "o1-mini-2024-09-12",
            "gpt-4o",
            "gpt-4o-2024-11-20",
            "gpt-4o-2024-08-06",
            "gpt-4o-2024-05-13",
            "gpt-4o-audio-preview",
            "gpt-4o-audio-preview-2024-10-01",
            "gpt-4o-audio-preview-2024-12-17",
            "gpt-4o-audio-preview-2025-06-03",
            "gpt-4o-mini-audio-preview",
            "gpt-4o-mini-audio-preview-2024-12-17",
            "gpt-4o-search-preview",
            "gpt-4o-mini-search-preview",
            "gpt-4o-search-preview-2025-03-11",
            "gpt-4o-mini-search-preview-2025-03-11",
            "chatgpt-4o-latest",
            "codex-mini-latest",
            "gpt-4o-mini",
            "gpt-4o-mini-2024-07-18",
            "gpt-4-turbo",
            "gpt-4-turbo-2024-04-09",
            "gpt-4-0125-preview",
            "gpt-4-turbo-preview",
            "gpt-4-1106-preview",
            "gpt-4-vision-preview",
            "gpt-4",
            "gpt-4-0314",
            "gpt-4-0613",
            "gpt-4-32k",
            "gpt-4-32k-0314",
            "gpt-4-32k-0613",
            "gpt-3.5-turbo",
            "gpt-3.5-turbo-16k",
            "gpt-3.5-turbo-0301",
            "gpt-3.5-turbo-0613",
            "gpt-3.5-turbo-1106",
            "gpt-3.5-turbo-0125",
            "gpt-3.5-turbo-16k-0613",
            "o1-pro",
            "o1-pro-2025-03-19",
            "o3-pro",
            "o3-pro-2025-06-10",
            "o3-deep-research",
            "o3-deep-research-2025-06-26",
            "o4-mini-deep-research",
            "o4-mini-deep-research-2025-06-26",
            "computer-use-preview",
            "computer-use-preview-2025-03-11",
            "gpt-5-codex",
            "gpt-5-pro",
            "gpt-5-pro-2025-10-06",
            "gpt-5.1-codex-max",
        ],
        str,
    ]
    """Model ID used to generate the response, like `gpt-4o` or `o3`.

    OpenAI offers a wide range of models with different capabilities, performance
    characteristics, and price points. Refer to the
    [model guide](https://platform.openai.com/docs/models) to browse and compare
    available models.
    """

    moderation: Optional[ResponseCreateModeration]
    """Configuration for running moderation on the input and output of this response."""

    multi_agent: Optional[ResponseCreateMultiAgent]
    """Configuration for server-hosted multi-agent execution."""

    parallel_tool_calls: Optional[bool]
    """Whether to allow the model to run tool calls in parallel."""

    previous_response_id: Optional[str]
    """The unique ID of the previous response to the model.

    Use this to create multi-turn conversations. Learn more about
    [conversation state](https://platform.openai.com/docs/guides/conversation-state).
    Cannot be used in conjunction with `conversation`.
    """

    prompt: Optional[BetaResponsePromptParam]
    """
    Reference to a prompt template and its variables.
    [Learn more](https://platform.openai.com/docs/guides/text?api-mode=responses#reusable-prompts).
    """

    prompt_cache_key: Optional[str]
    """
    Used by OpenAI to cache responses for similar requests to optimize your cache
    hit rates. Replaces the `user` field.
    [Learn more](https://platform.openai.com/docs/guides/prompt-caching).
    """

    prompt_cache_options: ResponseCreatePromptCacheOptions
    """Options for prompt caching.

    Supported for `gpt-5.6` and later models. By default, OpenAI automatically
    chooses one implicit cache breakpoint. You can add explicit breakpoints to
    content blocks with `prompt_cache_breakpoint`. Each request can write up to four
    breakpoints. For cache matching, OpenAI considers up to the latest 80
    breakpoints in the conversation, without a content-block lookback limit. Set
    `mode` to `explicit` to disable the implicit breakpoint. The `ttl` defaults to
    `30m`, which is currently the only supported value. See the
    [prompt caching guide](https://platform.openai.com/docs/guides/prompt-caching)
    for current details.
    """

    prompt_cache_retention: Optional[Literal["in_memory", "24h"]]
    """Deprecated. Use `prompt_cache_options.ttl` instead.

    The retention policy for the prompt cache. Set to `24h` to enable extended
    prompt caching, which keeps cached prefixes active for longer, up to a maximum
    of 24 hours.
    [Learn more](https://platform.openai.com/docs/guides/prompt-caching#prompt-cache-retention).
    This field expresses a maximum retention policy, while
    `prompt_cache_options.ttl` expresses a minimum cache lifetime. The two fields
    are independent and do not interact. For `gpt-5.5`, `gpt-5.5-pro`, and future
    models, only `24h` is supported.

    For older models that support both `in_memory` and `24h`, the default depends on
    your organization's data retention policy:

    - Organizations without ZDR enabled default to `24h`.
    - Organizations with ZDR enabled default to `in_memory` when
      `prompt_cache_retention` is not specified.
    """

    reasoning: Optional[ResponseCreateReasoning]
    """**gpt-5 and o-series models only**

    Configuration options for
    [reasoning models](https://platform.openai.com/docs/guides/reasoning).
    """

    safety_identifier: Optional[str]
    """
    A stable identifier used to help detect users of your application that may be
    violating OpenAI's usage policies. The IDs should be a string that uniquely
    identifies each user, with a maximum length of 64 characters. We recommend
    hashing their username or email address, in order to avoid sending us any
    identifying information.
    [Learn more](https://platform.openai.com/docs/guides/safety-best-practices#safety-identifiers).
    """

    service_tier: Optional[Literal["auto", "default", "flex", "scale", "priority", "fast"]]
    """Specifies the processing type used for serving the request.

    - If set to 'auto', then the request will be processed with the service tier
      configured in the Project settings. Unless otherwise configured, the Project
      will use 'default'.
    - If set to 'default', then the request will be processed with the standard
      pricing and performance for the selected model.
    - If set to '[flex](https://platform.openai.com/docs/guides/flex-processing)',
      then the request will be processed with the Flex Processing service tier.
    - To opt-in to [Fast mode](/api/docs/guides/fast-mode) at the request level,
      include the `service_tier=fast` or `service_tier=priority` parameter for
      Responses or Chat Completions. The response will show `service_tier=priority`
      regardless of if you specify `service_tier=fast` or `priority` in your
      request.
    - When not set, the default behavior is 'auto'.

    When the `service_tier` parameter is set, the response body will include the
    `service_tier` value based on the processing mode actually used to serve the
    request. This response value may be different from the value set in the
    parameter.
    """

    store: Optional[bool]
    """Whether to store the generated model response for later retrieval via API."""

    stream: Optional[bool]
    """
    If set to true, the model response data will be streamed to the client as it is
    generated using
    [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format).
    See the
    [Streaming section below](https://platform.openai.com/docs/api-reference/responses-streaming)
    for more information.
    """

    stream_options: Optional[ResponseCreateStreamOptions]
    """Options for streaming responses. Only set this when you set `stream: true`."""

    temperature: Optional[float]
    """What sampling temperature to use, between 0 and 2.

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic. We generally recommend altering
    this or `top_p` but not both.
    """

    text: BetaResponseTextConfigParam
    """Configuration options for a text response from the model.

    Can be plain text or structured JSON data. Learn more:

    - [Text inputs and outputs](https://platform.openai.com/docs/guides/text)
    - [Structured Outputs](https://platform.openai.com/docs/guides/structured-outputs)
    """

    tool_choice: ResponseCreateToolChoice
    """
    How the model should select which tool (or tools) to use when generating a
    response. See the `tools` parameter to see how to specify which tools the model
    can call.
    """

    tools: Iterable[BetaToolParam]
    """An array of tools the model may call while generating a response.

    You can specify which tool to use by setting the `tool_choice` parameter.

    We support the following categories of tools:

    - **Built-in tools**: Tools that are provided by OpenAI that extend the model's
      capabilities, like
      [web search](https://platform.openai.com/docs/guides/tools-web-search) or
      [file search](https://platform.openai.com/docs/guides/tools-file-search).
      Learn more about
      [built-in tools](https://platform.openai.com/docs/guides/tools).
    - **MCP Tools**: Integrations with third-party systems via custom MCP servers or
      predefined connectors such as Google Drive and SharePoint. Learn more about
      [MCP Tools](https://platform.openai.com/docs/guides/tools-connectors-mcp).
    - **Function calls (custom tools)**: Functions that are defined by you, enabling
      the model to call your own code with strongly typed arguments and outputs.
      Learn more about
      [function calling](https://platform.openai.com/docs/guides/function-calling).
      You can also use custom tools to call your own code.
    """

    top_logprobs: Optional[int]
    """
    An integer between 0 and 20 specifying the maximum number of most likely tokens
    to return at each token position, each with an associated log probability. In
    some cases, the number of returned tokens may be fewer than requested.
    """

    top_p: Optional[float]
    """
    An alternative to sampling with temperature, called nucleus sampling, where the
    model considers the results of the tokens with top_p probability mass. So 0.1
    means only the tokens comprising the top 10% probability mass are considered.

    We generally recommend altering this or `temperature` but not both.
    """

    truncation: Optional[Literal["auto", "disabled"]]
    """The truncation strategy to use for the model response.

    - `auto`: If the input to this Response exceeds the model's context window size,
      the model will truncate the response to fit the context window by dropping
      items from the beginning of the conversation.
    - `disabled` (default): If the input size will exceed the context window size
      for a model, the request will fail with a 400 error.
    """

    user: str
    """This field is being replaced by `safety_identifier` and `prompt_cache_key`.

    Use `prompt_cache_key` instead to maintain caching optimizations. A stable
    identifier for your end-users. Used to boost cache hit rates by better bucketing
    similar requests and to help OpenAI detect and prevent abuse.
    [Learn more](https://platform.openai.com/docs/guides/safety-best-practices#safety-identifiers).
    """


BetaResponsesClientEventParam: TypeAlias = Union[ResponseCreate, BetaResponseInjectEventParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ToolChoiceShell"]


class ToolChoiceShell(BaseModel):
    """Forces the model to call the shell tool when a tool call is required."""

    type: Literal["shell"]
    """The tool to call. Always `shell`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["VideoDownloadContentParams"]


class VideoDownloadContentParams(TypedDict, total=False):
    variant: Literal["video", "thumbnail", "spritesheet"]
    """Which downloadable asset to return. Defaults to the MP4 video."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import Literal

from ..._models import BaseModel
from .beta_response_input_item import BetaResponseInputItem

__all__ = ["BetaResponseInjectEvent"]


class BetaResponseInjectEvent(BaseModel):
    """
    Injects input items into an active response over a WebSocket connection.
    The items are validated and committed atomically. Currently, the server
    accepts client-owned tool outputs that resume a waiting agent.
    """

    input: List[BetaResponseInputItem]
    """Input items to inject into the active response."""

    response_id: str
    """The ID of the active response that should receive the input."""

    type: Literal["response.inject"]
    """The event discriminator. Always `response.inject`."""



from __future__ import annotations

from ._workload import (
    WorkloadIdentity as WorkloadIdentity,
    SubjectTokenProvider as SubjectTokenProvider,
    WorkloadIdentityAuth as WorkloadIdentityAuth,
    gcp_id_token_provider as gcp_id_token_provider,
    k8s_service_account_token_provider as k8s_service_account_token_provider,
    azure_managed_identity_token_provider as azure_managed_identity_token_provider,
)

__all__ = [
    "SubjectTokenProvider",
    "WorkloadIdentity",
    "WorkloadIdentityAuth",
    "k8s_service_account_token_provider",
    "azure_managed_identity_token_provider",
    "gcp_id_token_provider",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import FileTypes

__all__ = ["VideoEditParams", "Video", "VideoVideoReferenceInputParam"]


class VideoEditParams(TypedDict, total=False):
    prompt: Required[str]
    """Text prompt that describes how to edit the source video."""

    video: Required[Video]
    """Reference to the completed video to edit."""


class VideoVideoReferenceInputParam(TypedDict, total=False):
    """Reference to the completed video to edit."""

    id: Required[str]
    """The identifier of the completed video."""


Video: TypeAlias = Union[FileTypes, VideoVideoReferenceInputParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .webhooks import Webhooks as _Webhooks, AsyncWebhooks as _AsyncWebhooks


class Webhooks(_Webhooks):
    pass


class AsyncWebhooks(_AsyncWebhooks):
    pass


__all__ = ["Webhooks", "AsyncWebhooks"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = ["ResponseFunctionToolCall", "Caller", "CallerDirect", "CallerProgram"]


class CallerDirect(BaseModel):
    type: Literal["direct"]


class CallerProgram(BaseModel):
    caller_id: str
    """The call ID of the program item that produced this tool call."""

    type: Literal["program"]


Caller: TypeAlias = Annotated[Union[CallerDirect, CallerProgram, None], PropertyInfo(discriminator="type")]


class ResponseFunctionToolCall(BaseModel):
    """A tool call to run a function.

    See the
    [function calling guide](https://platform.openai.com/docs/guides/function-calling) for more information.
    """

    arguments: str
    """A JSON string of the arguments to pass to the function."""

    call_id: str
    """The unique ID of the function tool call generated by the model."""

    name: str
    """The name of the function to run."""

    type: Literal["function_call"]
    """The type of the function tool call. Always `function_call`."""

    id: Optional[str] = None
    """The unique ID of the function tool call."""

    caller: Optional[Caller] = None
    """The execution context that produced this tool call."""

    namespace: Optional[str] = None
    """The namespace of the function to run."""

    status: Optional[Literal["in_progress", "completed", "incomplete"]] = None
    """The status of the item.

    One of `in_progress`, `completed`, or `incomplete`. Populated when items are
    returned via API.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from openai import OpenAI, AsyncOpenAI
from tests.utils import assert_matches_type
from openai.types.beta import (
    BetaResponse,
    BetaCompactedResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestResponses:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: OpenAI) -> None:
        response = client.beta.responses.create()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: OpenAI) -> None:
        response = client.beta.responses.create(
            background=True,
            context_management=[
                {
                    "type": "type",
                    "compact_threshold": 1000,
                }
            ],
            conversation="string",
            include=["file_search_call.results"],
            input="string",
            instructions="instructions",
            max_output_tokens=16,
            max_tool_calls=0,
            metadata={"foo": "string"},
            model="gpt-5.1",
            moderation={
                "model": "model",
                "policy": {
                    "input": {"mode": "score"},
                    "output": {"mode": "score"},
                },
            },
            multi_agent={
                "enabled": True,
                "max_concurrent_subagents": 1,
            },
            parallel_tool_calls=True,
            previous_response_id="previous_response_id",
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt-cache-key-1234",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            safety_identifier="safety-identifier-1234",
            service_tier="auto",
            store=True,
            stream=False,
            stream_options={"include_obfuscation": True},
            temperature=1,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            top_logprobs=0,
            top_p=1,
            truncation="auto",
            user="user-1234",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: OpenAI) -> None:
        http_response = client.beta.responses.with_raw_response.create()

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.create() as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: OpenAI) -> None:
        response_stream = client.beta.responses.create(
            stream=True,
        )
        response_stream.response.close()

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: OpenAI) -> None:
        response_stream = client.beta.responses.create(
            stream=True,
            background=True,
            context_management=[
                {
                    "type": "type",
                    "compact_threshold": 1000,
                }
            ],
            conversation="string",
            include=["file_search_call.results"],
            input="string",
            instructions="instructions",
            max_output_tokens=16,
            max_tool_calls=0,
            metadata={"foo": "string"},
            model="gpt-5.1",
            moderation={
                "model": "model",
                "policy": {
                    "input": {"mode": "score"},
                    "output": {"mode": "score"},
                },
            },
            multi_agent={
                "enabled": True,
                "max_concurrent_subagents": 1,
            },
            parallel_tool_calls=True,
            previous_response_id="previous_response_id",
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt-cache-key-1234",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            safety_identifier="safety-identifier-1234",
            service_tier="auto",
            store=True,
            stream_options={"include_obfuscation": True},
            temperature=1,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            top_logprobs=0,
            top_p=1,
            truncation="auto",
            user="user-1234",
            betas=["responses_multi_agent=v1"],
        )
        response_stream.response.close()

    @parametrize
    def test_raw_response_create_overload_2(self, client: OpenAI) -> None:
        response = client.beta.responses.with_raw_response.create(
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_create_overload_2(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.create(
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve_overload_1(self, client: OpenAI) -> None:
        response = client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_method_retrieve_with_all_params_overload_1(self, client: OpenAI) -> None:
        response = client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            include=["file_search_call.results"],
            include_obfuscation=True,
            starting_after=0,
            stream=False,
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_raw_response_retrieve_overload_1(self, client: OpenAI) -> None:
        http_response = client.beta.responses.with_raw_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_streaming_response_retrieve_overload_1(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_overload_1(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            client.beta.responses.with_raw_response.retrieve(
                response_id="",
            )

    @parametrize
    def test_method_retrieve_overload_2(self, client: OpenAI) -> None:
        response_stream = client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        )
        response_stream.response.close()

    @parametrize
    def test_method_retrieve_with_all_params_overload_2(self, client: OpenAI) -> None:
        response_stream = client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
            include=["file_search_call.results"],
            include_obfuscation=True,
            starting_after=0,
            betas=["responses_multi_agent=v1"],
        )
        response_stream.response.close()

    @parametrize
    def test_raw_response_retrieve_overload_2(self, client: OpenAI) -> None:
        response = client.beta.responses.with_raw_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @parametrize
    def test_streaming_response_retrieve_overload_2(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve_overload_2(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            client.beta.responses.with_raw_response.retrieve(
                response_id="",
                stream=True,
            )

    @parametrize
    def test_method_delete(self, client: OpenAI) -> None:
        response = client.beta.responses.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert response is None

    @parametrize
    def test_method_delete_with_all_params(self, client: OpenAI) -> None:
        response = client.beta.responses.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            betas=["responses_multi_agent=v1"],
        )
        assert response is None

    @parametrize
    def test_raw_response_delete(self, client: OpenAI) -> None:
        http_response = client.beta.responses.with_raw_response.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert response is None

    @parametrize
    def test_streaming_response_delete(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert response is None

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            client.beta.responses.with_raw_response.delete(
                response_id="",
            )

    @parametrize
    def test_method_cancel(self, client: OpenAI) -> None:
        response = client.beta.responses.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_method_cancel_with_all_params(self, client: OpenAI) -> None:
        response = client.beta.responses.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_raw_response_cancel(self, client: OpenAI) -> None:
        http_response = client.beta.responses.with_raw_response.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    def test_streaming_response_cancel(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    def test_path_params_cancel(self, client: OpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            client.beta.responses.with_raw_response.cancel(
                response_id="",
            )

    @parametrize
    def test_method_compact(self, client: OpenAI) -> None:
        response = client.beta.responses.compact(
            model="gpt-5.6-sol",
        )
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    def test_method_compact_with_all_params(self, client: OpenAI) -> None:
        response = client.beta.responses.compact(
            model="gpt-5.6-sol",
            input="string",
            instructions="instructions",
            previous_response_id="resp_123",
            prompt_cache_key="prompt_cache_key",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            service_tier="auto",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    def test_raw_response_compact(self, client: OpenAI) -> None:
        http_response = client.beta.responses.with_raw_response.compact(
            model="gpt-5.6-sol",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    def test_streaming_response_compact(self, client: OpenAI) -> None:
        with client.beta.responses.with_streaming_response.compact(
            model="gpt-5.6-sol",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = http_response.parse()
            assert_matches_type(BetaCompactedResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True


class TestAsyncResponses:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.create()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.create(
            background=True,
            context_management=[
                {
                    "type": "type",
                    "compact_threshold": 1000,
                }
            ],
            conversation="string",
            include=["file_search_call.results"],
            input="string",
            instructions="instructions",
            max_output_tokens=16,
            max_tool_calls=0,
            metadata={"foo": "string"},
            model="gpt-5.1",
            moderation={
                "model": "model",
                "policy": {
                    "input": {"mode": "score"},
                    "output": {"mode": "score"},
                },
            },
            multi_agent={
                "enabled": True,
                "max_concurrent_subagents": 1,
            },
            parallel_tool_calls=True,
            previous_response_id="previous_response_id",
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt-cache-key-1234",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            safety_identifier="safety-identifier-1234",
            service_tier="auto",
            store=True,
            stream=False,
            stream_options={"include_obfuscation": True},
            temperature=1,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            top_logprobs=0,
            top_p=1,
            truncation="auto",
            user="user-1234",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        http_response = await async_client.beta.responses.with_raw_response.create()

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.create() as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        response_stream = await async_client.beta.responses.create(
            stream=True,
        )
        await response_stream.response.aclose()

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncOpenAI) -> None:
        response_stream = await async_client.beta.responses.create(
            stream=True,
            background=True,
            context_management=[
                {
                    "type": "type",
                    "compact_threshold": 1000,
                }
            ],
            conversation="string",
            include=["file_search_call.results"],
            input="string",
            instructions="instructions",
            max_output_tokens=16,
            max_tool_calls=0,
            metadata={"foo": "string"},
            model="gpt-5.1",
            moderation={
                "model": "model",
                "policy": {
                    "input": {"mode": "score"},
                    "output": {"mode": "score"},
                },
            },
            multi_agent={
                "enabled": True,
                "max_concurrent_subagents": 1,
            },
            parallel_tool_calls=True,
            previous_response_id="previous_response_id",
            prompt={
                "id": "id",
                "variables": {"foo": "string"},
                "version": "version",
            },
            prompt_cache_key="prompt-cache-key-1234",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            reasoning={
                "context": "auto",
                "effort": "none",
                "generate_summary": "auto",
                "mode": "standard",
                "summary": "auto",
            },
            safety_identifier="safety-identifier-1234",
            service_tier="auto",
            store=True,
            stream_options={"include_obfuscation": True},
            temperature=1,
            text={
                "format": {"type": "text"},
                "verbosity": "low",
            },
            tool_choice="none",
            tools=[
                {
                    "name": "name",
                    "parameters": {"foo": "bar"},
                    "strict": True,
                    "type": "function",
                    "allowed_callers": ["direct"],
                    "defer_loading": True,
                    "description": "description",
                    "output_schema": {"foo": "bar"},
                }
            ],
            top_logprobs=0,
            top_p=1,
            truncation="auto",
            user="user-1234",
            betas=["responses_multi_agent=v1"],
        )
        await response_stream.response.aclose()

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.with_raw_response.create(
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.create(
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve_overload_1(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_method_retrieve_with_all_params_overload_1(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            include=["file_search_call.results"],
            include_obfuscation=True,
            starting_after=0,
            stream=False,
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_raw_response_retrieve_overload_1(self, async_client: AsyncOpenAI) -> None:
        http_response = await async_client.beta.responses.with_raw_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve_overload_1(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_overload_1(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            await async_client.beta.responses.with_raw_response.retrieve(
                response_id="",
            )

    @parametrize
    async def test_method_retrieve_overload_2(self, async_client: AsyncOpenAI) -> None:
        response_stream = await async_client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        )
        await response_stream.response.aclose()

    @parametrize
    async def test_method_retrieve_with_all_params_overload_2(self, async_client: AsyncOpenAI) -> None:
        response_stream = await async_client.beta.responses.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
            include=["file_search_call.results"],
            include_obfuscation=True,
            starting_after=0,
            betas=["responses_multi_agent=v1"],
        )
        await response_stream.response.aclose()

    @parametrize
    async def test_raw_response_retrieve_overload_2(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.with_raw_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        await stream.close()

    @parametrize
    async def test_streaming_response_retrieve_overload_2(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.retrieve(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve_overload_2(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            await async_client.beta.responses.with_raw_response.retrieve(
                response_id="",
                stream=True,
            )

    @parametrize
    async def test_method_delete(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert response is None

    @parametrize
    async def test_method_delete_with_all_params(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            betas=["responses_multi_agent=v1"],
        )
        assert response is None

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncOpenAI) -> None:
        http_response = await async_client.beta.responses.with_raw_response.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert response is None

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.delete(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert response is None

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            await async_client.beta.responses.with_raw_response.delete(
                response_id="",
            )

    @parametrize
    async def test_method_cancel(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_method_cancel_with_all_params(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_raw_response_cancel(self, async_client: AsyncOpenAI) -> None:
        http_response = await async_client.beta.responses.with_raw_response.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaResponse, response, path=["response"])

    @parametrize
    async def test_streaming_response_cancel(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.cancel(
            response_id="resp_677efb5139a88190b512bc3fef8e535d",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(BetaResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True

    @parametrize
    async def test_path_params_cancel(self, async_client: AsyncOpenAI) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `response_id` but received ''"):
            await async_client.beta.responses.with_raw_response.cancel(
                response_id="",
            )

    @parametrize
    async def test_method_compact(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.compact(
            model="gpt-5.6-sol",
        )
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    async def test_method_compact_with_all_params(self, async_client: AsyncOpenAI) -> None:
        response = await async_client.beta.responses.compact(
            model="gpt-5.6-sol",
            input="string",
            instructions="instructions",
            previous_response_id="resp_123",
            prompt_cache_key="prompt_cache_key",
            prompt_cache_options={
                "mode": "implicit",
                "ttl": "30m",
            },
            prompt_cache_retention="in_memory",
            service_tier="auto",
            betas=["responses_multi_agent=v1"],
        )
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    async def test_raw_response_compact(self, async_client: AsyncOpenAI) -> None:
        http_response = await async_client.beta.responses.with_raw_response.compact(
            model="gpt-5.6-sol",
        )

        assert http_response.is_closed is True
        assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"
        response = http_response.parse()
        assert_matches_type(BetaCompactedResponse, response, path=["response"])

    @parametrize
    async def test_streaming_response_compact(self, async_client: AsyncOpenAI) -> None:
        async with async_client.beta.responses.with_streaming_response.compact(
            model="gpt-5.6-sol",
        ) as http_response:
            assert not http_response.is_closed
            assert http_response.http_request.headers.get("X-Stainless-Lang") == "python"

            response = await http_response.parse()
            assert_matches_type(BetaCompactedResponse, response, path=["response"])

        assert cast(Any, http_response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..... import _legacy_response
from ....._types import Body, Query, Headers, NotGiven, SequenceNotStr, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ....._base_client import make_request_options
from .....types.admin.organization.projects import model_permission_update_params
from .....types.admin.organization.projects.project_model_permissions import ProjectModelPermissions
from .....types.admin.organization.projects.project_model_permissions_deleted import ProjectModelPermissionsDeleted

__all__ = ["ModelPermissions", "AsyncModelPermissions"]


class ModelPermissions(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ModelPermissionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return ModelPermissionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ModelPermissionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return ModelPermissionsWithStreamingResponse(self)

    def retrieve(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissions:
        """
        Returns model permissions for a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._get(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissions,
        )

    def update(
        self,
        project_id: str,
        *,
        mode: Literal["allow_list", "deny_list"],
        model_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissions:
        """
        Updates model permissions for a project.

        Args:
          mode: The model permissions mode to apply.

          model_ids: The model IDs included in this permissions policy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._post(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            body=maybe_transform(
                {
                    "mode": mode,
                    "model_ids": model_ids,
                },
                model_permission_update_params.ModelPermissionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissions,
        )

    def delete(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissionsDeleted:
        """
        Deletes model permissions for a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return self._delete(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissionsDeleted,
        )


class AsyncModelPermissions(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncModelPermissionsWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncModelPermissionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncModelPermissionsWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncModelPermissionsWithStreamingResponse(self)

    async def retrieve(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissions:
        """
        Returns model permissions for a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._get(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissions,
        )

    async def update(
        self,
        project_id: str,
        *,
        mode: Literal["allow_list", "deny_list"],
        model_ids: SequenceNotStr[str],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissions:
        """
        Updates model permissions for a project.

        Args:
          mode: The model permissions mode to apply.

          model_ids: The model IDs included in this permissions policy.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._post(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            body=await async_maybe_transform(
                {
                    "mode": mode,
                    "model_ids": model_ids,
                },
                model_permission_update_params.ModelPermissionUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissions,
        )

    async def delete(
        self,
        project_id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ProjectModelPermissionsDeleted:
        """
        Deletes model permissions for a project.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not project_id:
            raise ValueError(f"Expected a non-empty value for `project_id` but received {project_id!r}")
        return await self._delete(
            path_template("/organization/projects/{project_id}/model_permissions", project_id=project_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                security={"admin_api_key_auth": True},
            ),
            cast_to=ProjectModelPermissionsDeleted,
        )


class ModelPermissionsWithRawResponse:
    def __init__(self, model_permissions: ModelPermissions) -> None:
        self._model_permissions = model_permissions

        self.retrieve = _legacy_response.to_raw_response_wrapper(
            model_permissions.retrieve,
        )
        self.update = _legacy_response.to_raw_response_wrapper(
            model_permissions.update,
        )
        self.delete = _legacy_response.to_raw_response_wrapper(
            model_permissions.delete,
        )


class AsyncModelPermissionsWithRawResponse:
    def __init__(self, model_permissions: AsyncModelPermissions) -> None:
        self._model_permissions = model_permissions

        self.retrieve = _legacy_response.async_to_raw_response_wrapper(
            model_permissions.retrieve,
        )
        self.update = _legacy_response.async_to_raw_response_wrapper(
            model_permissions.update,
        )
        self.delete = _legacy_response.async_to_raw_response_wrapper(
            model_permissions.delete,
        )


class ModelPermissionsWithStreamingResponse:
    def __init__(self, model_permissions: ModelPermissions) -> None:
        self._model_permissions = model_permissions

        self.retrieve = to_streamed_response_wrapper(
            model_permissions.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            model_permissions.update,
        )
        self.delete = to_streamed_response_wrapper(
            model_permissions.delete,
        )


class AsyncModelPermissionsWithStreamingResponse:
    def __init__(self, model_permissions: AsyncModelPermissions) -> None:
        self._model_permissions = model_permissions

        self.retrieve = async_to_streamed_response_wrapper(
            model_permissions.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            model_permissions.update,
        )
        self.delete = async_to_streamed_response_wrapper(
            model_permissions.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["CallRejectParams"]


class CallRejectParams(TypedDict, total=False):
    status_code: int
    """SIP response code to send back to the caller.

    Defaults to `603` (Decline) when omitted.
    """
